<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://www.linkedin.com/in/sajmirdoko/
 * @since      1.0.0
 *
 * @package    Woo_Additional_Checkout_Fees
 * @subpackage Woo_Additional_Checkout_Fees/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Woo_Additional_Checkout_Fees
 * @subpackage Woo_Additional_Checkout_Fees/includes
 * @author     sajdoko <sajdoko@gmail.com>
 */
class Woo_Additional_Checkout_Fees {

    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Woo_Additional_Checkout_Fees_Loader    $loader    Maintains and registers all hooks for the plugin.
     */
    protected $loader;

    /**
     * The unique identifier of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $plugin_name    The string used to uniquely identify this plugin.
     */
    protected $plugin_name;

    /**
     * The current version of the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $version    The current version of the plugin.
     */
    protected $version;

    /**
     * Define the core functionality of the plugin.
     *
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     * Load the dependencies, define the locale, and set the hooks for the admin area and
     * the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function __construct() {
        if (defined('WOO_ADDITIONAL_CHECKOUT_FEES_VERSION')) {
            $this->version = WOO_ADDITIONAL_CHECKOUT_FEES_VERSION;
        } else {
            $this->version = '1.0.0';
        }
        $this->plugin_name = 'woo-additional-checkout-fees';

        $this->load_dependencies();
        $this->set_locale();
        $this->define_admin_hooks();
        $this->define_public_hooks();

    }

    /**
     * Load the required dependencies for this plugin.
     *
     * Include the following files that make up the plugin:
     *
     * - Woo_Additional_Checkout_Fees_Loader. Orchestrates the hooks of the plugin.
     * - Woo_Additional_Checkout_Fees_i18n. Defines internationalization functionality.
     * - Woo_Additional_Checkout_Fees_Admin. Defines all hooks for the admin area.
     * - Woo_Additional_Checkout_Fees_Public. Defines all hooks for the public side of the site.
     *
     * Create an instance of the loader which will be used to register the hooks
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function load_dependencies() {

        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-woo-additional-checkout-fees-loader.php';

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-woo-additional-checkout-fees-i18n.php';

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-woo-additional-checkout-fees-admin.php';

        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'public/class-woo-additional-checkout-fees-public.php';

        $this->loader = new Woo_Additional_Checkout_Fees_Loader();

    }

    /**
     * Define the locale for this plugin for internationalization.
     *
     * Uses the Woo_Additional_Checkout_Fees_i18n class in order to set the domain and to register the hook
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function set_locale() {

        $plugin_i18n = new Woo_Additional_Checkout_Fees_i18n();

        $this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');

    }

    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_admin_hooks() {

        $plugin_admin = new Woo_Additional_Checkout_Fees_Admin($this->get_plugin_name(), $this->get_version());

        if ( ! $this->is_woocommerce_activated() ) {

            $this->loader->add_action('admin_init', $plugin_admin, 'additional_checkout_fees_disable_plugin');
            $this->loader->add_action('admin_notices', $plugin_admin, 'additional_checkout_fees_show_notice');

        } else {

            $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
            $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');

            // Add menu item
            $this->loader->add_action('admin_menu', $plugin_admin, 'additional_checkout_fees_add_admin_menu', 99);

            // Save/Update our plugin options
            $this->loader->add_action('admin_init', $plugin_admin, 'additional_checkout_fees_options_update');

            // Add Settings link to the plugin
            $plugin_basename = plugin_basename(plugin_dir_path(__DIR__) . $this->plugin_name . '.php');
            $this->loader->add_filter('plugin_action_links_' . $plugin_basename, $plugin_admin, 'additional_checkout_fees_add_action_links');

        }
    }

    /**
     * Register all of the hooks related to the public-facing functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_public_hooks() {

        $plugin_public = new Woo_Additional_Checkout_Fees_Public($this->get_plugin_name(), $this->get_version());

        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles');
        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');

        $this->loader->add_action('woocommerce_review_order_before_payment', $plugin_public, 'additional_checkout_fees_choice');
        $this->loader->add_action('woocommerce_cart_calculate_fees', $plugin_public, 'additional_checkout_fees_choice_fee', 20, 1);
        $this->loader->add_action('wp_footer', $plugin_public, 'additional_checkout_fees_choice_refresh');
        $this->loader->add_action('wp_ajax_woo_get_ajax_data', $plugin_public, 'additional_checkout_fees_choice_set_session');
        $this->loader->add_action('wp_ajax_nopriv_woo_get_ajax_data', $plugin_public, 'additional_checkout_fees_choice_set_session');
    }

    /**
     * Run the loader to execute all of the hooks with WordPress.
     *
     * @since    1.0.0
     */
    public function run() {
        $this->loader->run();
    }

    /**
     * The name of the plugin used to uniquely identify it within the context of
     * WordPress and to define internationalization functionality.
     *
     * @since     1.0.0
     * @return    string    The name of the plugin.
     */
    public function get_plugin_name() {
        return $this->plugin_name;
    }

    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     *
     * @since     1.0.0
     * @return    Woo_Additional_Checkout_Fees_Loader    Orchestrates the hooks of the plugin.
     */
    public function get_loader() {
        return $this->loader;
    }

    /**
     * Retrieve the version number of the plugin.
     *
     * @since     1.0.0
     * @return    string    The version number of the plugin.
     */
    public function get_version() {
        return $this->version;
    }

    /**
     * Check if WooCommerce is activated
     *
     * @since     1.0.1
     * @return    boolean    True or false.
     */
    public function is_woocommerce_activated() {
        return in_array( 'woocommerce/woocommerce.php', (array) get_option( 'active_plugins', array() ) );
    }

}
